import os # rozne systemove volania
import time # praca s casom
from threading import Thread # paralelizmus zalozeny na vlaknach

import dash # Dash framework
from dash import html
from dash import dcc
from dash.dependencies import Input,Output

from scapy.all import * # Scapy - send, sniff, dissect, forge network packets

import pandas as pd # kniznica Pandas
import numpy as np # kniznica NumPy

import plotly.graph_objects as go # kniznica Plotly
import plotly.express as px

# inicializacia Pandas DataFrame
net_data = pd.DataFrame(columns = ["netid","ID","TYPE","MAC","SSID","SR","VENDOR"])
# index = netid
net_data.set_index("netid",inplace = True)

# aktualny kanal
act_channel = 1
# aktualny index ramca
act_frame = 0


# callable object - zmena WiFi kanalu
def my_hopper(iface):
    global act_channel
    while True:
        # vykonanie systemoveho prikazu
        os.system('iwconfig %s channel %d' % (iface,act_channel))
        #print('aktualny kanal %d' % (act_ch))
        act_channel += 1
        if act_channel > 13:
            act_channel = 1
        time.sleep(0.5) # 0.5s pauza


# callable object - ziskanie informacii z prijatych ramcov
def my_scanner(iface):
    global act_frame
    while True:
        # zachytenie 5-tich Probe Request ramcov
        outstr = subprocess.check_output(['tshark -n -i wlp2s0mon -f "subtype probe-req" -T fields -e wlan.sa -e wlan.ssid -e wlan.supported_rates -e wlan.tag.oui -c 5'], shell=True).decode(sys.stdout.encoding)
        # rozdelenie retazca na jednotlive riadky
        splitoutstr = outstr.split('\n')

        for v in splitoutstr:
            # rozdelenie textu na 1 riadku s oddelovacom '\t'
            splitv = v.split('\t')
            # ak existuje 1. cast textu
            if splitv[0] != '':
                # MAC adresa = 1. cast
                MAC = str(splitv[0])
                try: # SSID = 2. cast
                    SSID = str(splitv[1])
                except:
                    SSID = ''
                try: # Supported Rates = 3. cast
                    SR = str(splitv[2])
                except:
                    SR = ''
                try: # Vendor OUI = 4. cast
                    VENDOR = splitv[3]
                    if VENDOR != '': # vyberie sa iba prva cast retazca pred ','
                        if ',' in VENDOR:
                            VENDOR = VENDOR[:VENDOR.find(',')]
                        # prevod int na 3 bytes
                        VENDOR = int(VENDOR).to_bytes(3,'big')
                        B1 = int(VENDOR[0]) 
                        B2 = int(VENDOR[1])
                        B3 = int(VENDOR[2])
                        # formatovanie 3 bytes na hex tvar
                        VENDOR = format(B1,'x')+' '+format(B2,'x')+' '+format(B3,'x')
                except:
                    VENDOR = ''

                if SSID == '':
                    SSID = '-'
                if SR == '':
                    SR = '-'
                if VENDOR == '':
                    VENDOR = '-'

                # pridanie zaznamu do DataFrame
                net_data.loc[act_frame] = (act_frame,'Probe Request',MAC,SSID,SR,VENDOR)

                # nove cislo zaznamu
                act_frame += 1

                # ak bolo prijatych viac ako 50 ramcov, DataFrame sa vymaze
                if act_frame > 50:
                    net_data.drop(net_data.index,inplace = True)
                    act_frame = 0

        # zachytenie 5-tich Beacon ramcov
        outstr = subprocess.check_output(['tshark -n -i wlp2s0mon -f "subtype beacon" -T fields -e wlan.sa -e wlan.ssid -e radiotap.channel.freq -e radiotap.dbm_antsignal -c 5'], shell=True).decode(sys.stdout.encoding)
        # rozdelenie retazca na jednotlive riadky
        splitoutstr = outstr.split('\n')

        for v in splitoutstr:
            # rozdelenie textu na 1 riadku s oddelovacom '\t'
            splitv = v.split('\t')
            # ak existuje 1. cast textu
            if splitv[0] != '':
                # MAC adresa = 1. cast
                MAC = str(splitv[0])
                try: # SSID = 2. cast
                    SSID = str(splitv[1])
                except:
                    SSID = ''
                try: # Frequency = 3. cast
                    FR = str(splitv[2])
                except:
                    FR = ''
                try: # Signal strength = 4. cast
                    SIG = str(splitv[3])
                    SIG = SIG[:SIG.find(',')] # vyberie sa iba prva cast retazca pred ','
                except:
                    SIG = ''

                if SSID == '':
                    SSID = '-'
                if FR == '':
                    FR = '-'
                if SIG == '':
                    SIG = '-'

                # pridanie zaznamu do DataFrame
                net_data.loc[act_frame] = (act_frame,'Beacon',MAC,SSID,FR,SIG)

                # nove cislo zaznamu
                act_frame += 1

                # ak bolo prijatych viac ako 50 ramcov, DataFrame sa vymaze
                if act_frame > 50:
                    net_data.drop(net_data.index,inplace = True)
                    act_frame = 0

# nazov WiFi (WLAN) rozhrania v nasom OS
mIface = 'wlp2s0'
mMAC = 'MAC'   # TBD
mIP = 'IP'     # TBD
mSSID = 'SSID' # TBD

# inicializacia Dash aplikacie
app = dash.Dash(__name__)

# definicia Dash layoutu
app.layout = html.Div(
    # 1 riadok
    children = [html.Div(className = 'row',
                         children = [# 1. stlpec
                                     html.Div(className = 'col',style = { 'margin' : 10 },
                                              children = [# 1 riadok
                                                          html.H3('WiFi Beacons / Probe Requests skener'),
                                                          html.Div(className = 'block_container',
                                                                   style = {'margin' : 20},
                                                                   children = ['Moje sieťové rozhranie: ',html.Div(className = 'DATAs',children = [mIface]),
                                                                               ', MAC adresa: ',html.Div(className = 'DATAs',children = [mMAC]),
                                                                               ', IP adresa: ',html.Div(className = 'DATAs',children = [mIP]),
                                                                               ', SSID: ',html.Div(className = 'DATAs',children = [mSSID])]
                                                          ),
                                                          dcc.Graph(id = 'live-update-table'),
                                                          dcc.Interval(
                                                              id = 'interval-table',
                                                              interval = 5000, # updatuje n_intervals kazdych 5s
                                                              n_intervals = 0
                                                          )
                                              ]
                                     )
                         ]
                )
    ]
)

# Callback funkcia pre tabulku
@app.callback(Output('live-update-table','figure'),[Input('interval-table','n_intervals')])
def update_table_object(n):
    # tabulka
    fig = go.Figure(data=[go.Table(header = dict(values = ['id','Type','MAC','SSID','Supported Rates / Frequency','Vendor OUI / Signal strength'],
                                                 fill_color = '#c86813'),
                                   cells = dict(values = [net_data.ID,net_data.TYPE,net_data.MAC,net_data.SSID,net_data.SR,net_data.VENDOR],
                                                fill_color = '#333333'),
                                   columnwidth = [20,100,100,180,180,140])
                         ]
                   )
    # nastavenie stylu tabulky
    fig.update_layout(template = 'plotly_dark',
                      paper_bgcolor = 'rgba(0,0,0,100)',
                      margin = {'l' : 10,'r' : 10,'t' : 10,'b' : 10},
                      showlegend = False)

    # aktualizacia dcc.Graph(figure)
    return fig


if __name__ == "__main__":
    # zmena WiFi kanalu v samostatnom vlakne
    hopper = Thread(target = my_hopper,args = (mIface+'mon',))
    hopper.daemon = True
    hopper.start()
    # ziskavanie informacii z prijatych ramcov v samostatnom vlakne
    scanner = Thread(target = my_scanner,args = (mIface+'mon',))
    scanner.daemon = True
    scanner.start()

    # spustenie Dash aplikacie
    app.run_server(debug = True)
